#Link CPs - Surface Variables (Iberian Peninsula and Catalunya)
#Plotting spatial patterns (anom)
library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
#library(matrixStats)
#library(metR)
library(RColorBrewer)

rm(list=ls()) ; gc()

#Directory information
setwd("/esarchive/scratch/molmo/")
OUTPUTS <- "/esarchive/scratch/molmo/"

#Useful functions
png.save <- function(x,filename,main,xlab,ylab){
  png(filename = paste(filename,".png",sep=""))
  plot(x,type="o",main=main,xlab = xlab,ylab=ylab)
  dev.off()
}

#######################---tasmax---####################################

do <- c("D1","D2","D3")

do <- c("D4")
seasons <- list("Summer"=c("06","07","08"),"Winter"=c("12","01","02"),
                "Spring"=c("03","04","05"),"Autumn"=c("09","10","11"),
                "Annual"=c("01","02","03","04","05","06",
                "07","08","09","10","11","12"))

for(sn in c(1:5)){
  message(names(seasons)[sn])
for(d in c(1)){
  dom <- do[d]
  for(k in c(8)){
    #Łoad observational data
    message("TASMAX")
    nc <- nc_open("tasmax-5km_5022_ok_anom.nc") #tasmax-5km_5022_ok.nc #"tasmax-5km_5022_anom.nc"
    nc_var <- ncvar_get(nc,"tasmax")
    nc_lon <- ncvar_get(nc,"lon")
    nc_lat <- ncvar_get(nc,"lat")
    
    # print(length(which(nc_var < -300)))
    # nc_var[which(nc_var < -300)] <- NA
    # print(length(which(nc_var > 500)))
    # nc_var[which(nc_var > 500)] <- NA
    
    calendario <- seq(as.Date("1951-01-01"),as.Date("2022-12-31"),by="day")
    fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
    
    #Selection of spatial domains
    gridsize_x <- abs(nc_lon[2]-nc_lon[1])
    gridsize_y <- abs(nc_lat[2]-nc_lat[1])
    
    domain1 <- list(lon <- which(nc_lon > -15 & nc_lon < 15),
                    lat <- which(nc_lat > 32 & nc_lat < 48)) #red domain D1 #Spain
    
    domain2 <- list(lon <- which(nc_lon > -10 & nc_lon < 25),
                    lat <- which(nc_lat > 33 & nc_lat < 45)) #red domain D2 PEP Mediterranean
    
    domain3 <- list(lon <- which(nc_lon > -35 & nc_lon < 35),
                    lat <- which(nc_lat > 20 & nc_lat < 70)) #red domain D3 #Europe
    
    #Load clustering information
    domain <- domain1 
    #domain <- domain1 ; k=7 ; dom = "D2" ; x1 = -10; x2 = 25; y1 = 33; y2 = 45
    #domain <- domain1 ; k=4; dom = "D3" ; x1 = -35 ; x2 = 35 ; y1 = 20 ; y2 = 70
    
    nc_var <- nc_var[domain[[1]],domain[[2]],] ; gc()
    
    #Organize and Standardize data
    message("Standardize")
    var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
    rm(nc) ; gc()
    #var <- t(scale(t(var),center=T,scale=T))
    var <- var[,which(format(calendario,"%Y")%in%c(1951:2020)& format(calendario,"%m")%in%seasons[[sn]])]
    rm(nc_var) ; gc()
    
    load(file = paste("WardCluster_Dendrogram_",dom,"_50km.Rdata",sep=""))
    calendario_ERA <- seq(as.Date("1950-01-01"),as.Date("2022-12-31"),by="day")
    vector_cluster <- cutree(ward_clustering,k)[which(format(calendario_ERA,"%Y")%in%c(1951:2020)& format(calendario_ERA,"%m")%in%seasons[[sn]])]
    
    #Preparing data
    message("Organizing data...")
    longitude=rep(nc_lon[domain[[1]]],length(domain[[2]]))
    latitude=rep(nc_lat[domain[[2]]],each=length(domain[[1]]))
    
    Z <- list() 
    
    for(cp in 1:k){
    message(paste("Significance of pattern ",cp,sep=""))
      sign <- NA
      for(grid in 1:dim(var)[1]){
        sign[grid] <- NA
        if(!is.na(mean(var[grid,which(vector_cluster==cp)],na.rm=T)) & var(var[grid,which(vector_cluster==cp)])!=0)
          sign[grid] <- t.test(var[grid,which(vector_cluster==cp)])$p.value
      }
      Z[[cp]] <- rowMeans(var[,which(vector_cluster==cp)],na.rm=T)
      Z[[cp]][which(sign>0.05)] <- NA
      
    }
    
    datitos <- data.frame(Longitude=rep(longitude,k),
                          Latitude=rep(latitude,k),
                          Z = unlist(Z),
                          CP = rep(1:k,each=length(unlist(Z))/k),
                          Domain=rep(dom,length(unlist(Z))))
    
    datitos$CP <- factor(datitos$CP,levels=c(1:k))
    
    
    # p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=3)+ 
    #   scale_fill_gradientn(name=expression("Temperature (ºC)"),breaks=seq(-5,5,by=2),limits=c(-5,5),
    #                        colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
    #                        na.value = "white") +
    #   coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
    #   xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("Tasmax Anom Patterns") 
    # p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
    # p <- p + geom_rect(aes(xmin=0,xmax=4,ymin=40,ymax=43),col="black",fill="grey",alpha=0.01)
    
    save(datitos,file=paste("SpatialPatterns_Ward_Surface_TX_DailyAnom_",dom,"_Iberia_",k,"_",names(seasons)[sn],"_AEMET-5km_CPs.Rdata",sep=""))
    
    # p <- ggplot(subset(datitos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=3)+ 
    #   scale_fill_gradientn(name=expression("Temperature (ºC)"),breaks=seq(-5,5,by=2),limits=c(-5,5),
    #                        colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
    #                        na.value = "white") +
    #   coord_quickmap(xlim=c(0,4),ylim=c(40,43),expand = F) +
    #   xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("Tasmax Anom Patterns") 
    # p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
    
    #save(datitos,file=paste("SpatialPatterns_Ward_Surface_TX_Anom_",dom,"_Catalunya_",k,"_AEMET-5km_CPs.Rdata",sep=""))
    
    ####################################################################################
    #######################---tasmin---####################################
    rm(datitos,nc,p,var,Z) ; gc()
    
    message("TASMIN")
    #Łoad observational data
    nc <- nc_open("tasmin-5km_5020_ok_anom.nc")
    nc_var <- ncvar_get(nc,"tasmin")
    nc_lon <- ncvar_get(nc,"lon")
    nc_lat <- ncvar_get(nc,"lat")
    
    calendario <- seq(as.Date("1951-01-01"),as.Date("2020-12-31"),by="day")
    fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
    
    #Selection of spatial domains
    gridsize_x <- abs(nc_lon[2]-nc_lon[1])
    gridsize_y <- abs(nc_lat[2]-nc_lat[1])
    
    domain1 <- list(lon <- which(nc_lon > -15 & nc_lon < 15),
                    lat <- which(nc_lat > 32 & nc_lat < 48)) #red domain D1 #Spain
    
    domain2 <- list(lon <- which(nc_lon > -10 & nc_lon < 25),
                    lat <- which(nc_lat > 33 & nc_lat < 45)) #red domain D2 PEP Mediterranean
    
    domain3 <- list(lon <- which(nc_lon > -35 & nc_lon < 35),
                    lat <- which(nc_lat > 20 & nc_lat < 70)) #red domain D3 #Europe
    
    #Load clustering information
    domain <- domain1 #; k=5 ; dom = "D1" ; x1 = -15 ; x2 = 15 ; y1 = 32 ; y2 = 48
    #domain <- domain1 ; k=7 ; dom = "D2" ; x1 = -10; x2 = 25; y1 = 33; y2 = 45
    #domain <- domain1 ; k=4; dom = "D3" ; x1 = -35 ; x2 = 35 ; y1 = 20 ; y2 = 70
    
    nc_var <- nc_var[domain[[1]],domain[[2]],] ; gc()
    
    #Organize and Standardize data
     var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
    rm(nc) ; gc()
   # var <- t(scale(t(var),center=T,scale=T))
    var <- var[,which(format(calendario,"%Y")%in%c(1951:2020)& format(calendario,"%m")%in%seasons[[sn]])]
    rm(nc_var) ; gc()
    
    load(file = paste("WardCluster_Dendrogram_",dom,"_50km.Rdata",sep=""))
    calendario_ERA <- seq(as.Date("1950-01-01"),as.Date("2022-12-31"),by="day")
    vector_cluster <- cutree(ward_clustering,k)[which(format(calendario_ERA,"%Y")%in%c(1951:2020)& format(calendario_ERA,"%m")%in%seasons[[sn]])]
    
    #Preparing data
    longitude=rep(nc_lon[domain[[1]]],length(domain[[2]]))
    latitude=rep(nc_lat[domain[[2]]],each=length(domain[[1]]))
    
    Z <- list() 
    
    for(cp in 1:k){
        message(paste("Significance of pattern ",cp,sep=""))
      sign <- NA
      for(grid in 1:dim(var)[1]){
        sign[grid] <- NA
        if(!is.na(mean(var[grid,which(vector_cluster==cp)],na.rm=T))& var(var[grid,which(vector_cluster==cp)])!=0)
          sign[grid] <- t.test(var[grid,which(vector_cluster==cp)])$p.value
      }
      Z[[cp]] <- rowMeans(var[,which(vector_cluster==cp)],na.rm=T)
      Z[[cp]][which(sign>0.05)] <- NA
      
    }
    
    datitos <- data.frame(Longitude=rep(longitude,k),
                          Latitude=rep(latitude,k),
                          Z = unlist(Z),
                          CP = rep(1:k,each=length(unlist(Z))/k),
                          Domain=rep(dom,length(unlist(Z))))
    datitos$CP <- factor(datitos$CP,levels=c(1:k))
    
    
    # p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=3)+ 
    #   scale_fill_gradientn(name=expression("Temperature (ºC)"),breaks=seq(-5,5,by=2),limits=c(-5,5),
    #                        colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
    #                        na.value = "white") +
    #   coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
    #   xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("Tasmin Anom Patterns") 
    # p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
    # p <- p + geom_rect(aes(xmin=0,xmax=4,ymin=40,ymax=43),col="black",fill="grey",alpha=0.01)
    
    save(datitos,file=paste("SpatialPatterns_Ward_Surface_TN_DailyAnom_",dom,"_Iberia_",k,"_",names(seasons)[sn],"_AEMET-5km_CPs.Rdata",sep=""))
    
    # p <- ggplot(subset(datitos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=3)+ 
    #   scale_fill_gradientn(name=expression("Temperature (ºC)"),breaks=seq(-5,5,by=2),limits=c(-5,5),
    #                        colours=rev(brewer.pal(9,"RdBu")),aesthetics=c("fill"),
    #                        na.value = "white") +
    #   coord_quickmap(xlim=c(0,4),ylim=c(40,43),expand = F) +
    #   xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("Tasmin Anom Patterns") 
    # p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
    
    #save(datitos,file=paste("SpatialPatterns_Ward_Surface_TN_Anom_",dom,"_Catalunya_",k,"_AEMET-5km_CPs.Rdata",sep=""))
    
    #################---precip---##############################################
    rm(datitos,nc,p,var,Z) ; gc()
    
    message("PRECIP")
    
    #Łoad observational data
    nc <- nc_open("prlr-5km_5021_mm_ok_anom.nc")
    nc_var <- ncvar_get(nc,"prlr")
    nc_lon <- ncvar_get(nc,"lon")
    nc_lat <- ncvar_get(nc,"lat")
    
    calendario <- seq(as.Date("1951-01-01"),as.Date("2021-12-31"),by="day")
    fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
    
    #Selection of spatial domains
    gridsize_x <- abs(nc_lon[2]-nc_lon[1])
    gridsize_y <- abs(nc_lat[2]-nc_lat[1])
    
    domain1 <- list(lon <- which(nc_lon > -15 & nc_lon < 15),
                    lat <- which(nc_lat > 32 & nc_lat < 48)) #red domain D1 #Spain
    
    domain2 <- list(lon <- which(nc_lon > -10 & nc_lon < 25),
                    lat <- which(nc_lat > 33 & nc_lat < 45)) #red domain D2 PEP Mediterranean
    
    domain3 <- list(lon <- which(nc_lon > -35 & nc_lon < 35),
                    lat <- which(nc_lat > 20 & nc_lat < 70)) #red domain D3 #Europe
    
    #Load clustering information
    domain <- domain1 #; k=5; dom = "D1" ; x1 = -15 ; x2 = 15 ; y1 = 32 ; y2 = 48
    #domain <- domain1 ; k=7 ; dom = "D2" ; x1 = -10; x2 = 25; y1 = 33; y2 = 45
    #domain <- domain1 ; k=4; dom = "D3" ; x1 = -35 ; x2 = 35 ; y1 = 20 ; y2 = 70
    
    nc_var <- nc_var[domain[[1]],domain[[2]],] ; gc()
    
    #Organize and Standardize data
   var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
    rm(nc) ; gc()
   # var <- t(scale(t(var),center=T,scale=T))
    var <- var[,which(format(calendario,"%Y")%in%c(1951:2020)& format(calendario,"%m")%in%seasons[[sn]])]
    rm(nc_var) ; gc()
    
    load(file = paste("WardCluster_Dendrogram_",dom,"_50km.Rdata",sep=""))
    calendario_ERA <- seq(as.Date("1950-01-01"),as.Date("2022-12-31"),by="day")
    vector_cluster <- cutree(ward_clustering,k)[which(format(calendario_ERA,"%Y")%in%c(1951:2020)& format(calendario_ERA,"%m")%in%seasons[[sn]])]
    
    #Preparing data
    longitude=rep(nc_lon[domain[[1]]],length(domain[[2]]))
    latitude=rep(nc_lat[domain[[2]]],each=length(domain[[1]]))
    
    Z <- list() 
    
    for(cp in 1:k){
        message(paste("Significance of pattern ",cp,sep=""))
      sign <- NA
      for(grid in 1:dim(var)[1]){
        sign[grid] <- NA
        if(!is.na(mean(var[grid,which(vector_cluster==cp)],na.rm=T)) & var(var[grid,which(vector_cluster==cp)])!=0)
          sign[grid] <- t.test(var[grid,which(vector_cluster==cp)])$p.value
      } 
      Z[[cp]] <- rowMeans(var[,which(vector_cluster==cp)],na.rm=T)
      Z[[cp]][which(sign>0.05)] <- NA
    }
    
    datitos <- data.frame(Longitude=rep(longitude,k),
                          Latitude=rep(latitude,k),
                          Z = unlist(Z),
                          CP = rep(1:k,each=length(unlist(Z))/k),
                          Domain=rep(dom,length(unlist(Z))))
    datitos$CP <- factor(datitos$CP,levels=c(1:k))
    
    
    # p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=3)+ 
    #   scale_fill_gradientn(name=expression("Precipitation (mm)"),breaks=seq(-5,5,by=2),limits=c(-5,5),
    #                        colours=brewer.pal(9,"BrBG"),aesthetics=c("fill"),
    #                        na.value = "white") +
    #   coord_quickmap(xlim=c(min(datitos$Longitude),max(datitos$Longitude)),ylim=c(min(datitos$Latitude),max(datitos$Latitude)),expand = F) +
    #   xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("Precip Anom Patterns") 
    # p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
    # p <- p + geom_rect(aes(xmin=0,xmax=4,ymin=40,ymax=43),col="black",fill="grey",alpha=0.01)
    
    save(datitos,file=paste("SpatialPatterns_Ward_Surface_PR_DailyAnom_",dom,"_Iberia_",k,"_",names(seasons)[sn],"_AEMET-5km_CPs.Rdata",sep=""))
    # p <- ggplot(subset(datitos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),aes(x=Longitude,y=Latitude))  + geom_tile(aes(fill=Z))  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=3)+ 
    #   scale_fill_gradientn(name=expression("Precipitation (mm)"),breaks=seq(-5,5,by=2),limits=c(-5,5),
    #                        colours=brewer.pal(9,"BrBG"),aesthetics=c("fill"),
    #                        na.value = "white") +
    #   coord_quickmap(xlim=c(0,4),ylim=c(40,43),expand = F) +
    #   xlab("Longitude") + ylab("Latitude") + theme_bw() + ggtitle("Precip Anom Patterns") 
    # p <- p + geom_rect(aes(xmin=x1,xmax=x2,ymin=y1,ymax=y2),col="red",fill=NA)
   rm(datitos) ; gc() 
    #save(datitos,file=paste("SpatialPatterns_Ward_Surface_PR_Anom_",dom,"_Catalunya_",k,"_AEMET-5km_CPs.Rdata",sep=""))
  }
}
}
