library(ggplot2)
library(ncdf4)
library(trend)
library(loadeR)
library(transformeR)
library(visualizeR)
library(RColorBrewer)

rm(list=ls()) ; gc()

#Working directory
dir <- "/esarchive/scratch/molmo/CLIMCAT_MJ/CMIP6/"
OUTPUTS <- "/esarchive/scratch/molmo/CLIMCAT_MJ/OUTPUTS/"
setwd(dir)

invariable <- loadGridData(dataset = "/esarchive/scratch/molmo/mask_1_Med_bil.nc",
                           var="precip") %>% binaryGrid(condition = "GE",threshold=0)
scenario <- c("ssp126","ssp245","ssp585")

for(sc in 1:length(scenario)){
  
#-------Annual indices---------#
#Useful information
indices <- c("wsdi","csdi","tr",
             "r20mm","prcptot","cdd","cwd")
files <- intersect(list.files(pattern="2100.nc",recursive = T),list.files(pattern="yr",recursive = T))
files <- intersect(files,list.files(pattern=scenario[sc],recursive=T))

ssn <- "Annual"
seasons <- list(c(1:12))
Model <- NA

series.indices <- list()

for(i in 1:length(indices)){
  print(indices[i])
  files_subset <- intersect(files,list.files(pattern=indices[i],recursive=T))
  series.indices[[i]] <- list()
  
  for(m in 1:length(files_subset)){
    print(files_subset[m])
    
    series.indices[[i]][[m]] <- list()
    
    for(s in 1:length(seasons)){
      di <- dataInventory(files_subset[m])
      
      ini <- nchar(names(di))                            
      out <- nchar("_historical-ssp585_r1i1p1f1_1950-2100.nc") 
      Model[m] <- substr(files_subset[m],start=ini+32,stop=nchar(files_subset[m])-out)
      
      data <- loadGridData(files_subset[m],var=names(di),years=1950:2100)
      
      #Interpolate into a common grid and mask ocean
      data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
      for(y in 1:dim(data$Data)[1]){
        data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
      }
      
      #Estimating anomalies wrt 1981-2010 reference period
      data <- scaleGrid(data,base = subsetGrid(data,years=1981:2010),type = "center")
      
      #Serie regional
      EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
      Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
      
      Serie <- data.frame(Serie=c(EuroMediterranean$Data,Catalonia$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Region=rep(c("EuroMediterranean","Catalonia"),each=length(getYearsAsINDEX(Catalonia))),
                          Year=rep(getYearsAsINDEX(Catalonia),2),
                          Tau=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)[["estimates"]][["tau"]],
                                    mk.test(na.omit(Catalonia$Data),continuity = T)[["estimates"]][["tau"]]),each=length(getYearsAsINDEX(Catalonia))),
                          pv=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)$p.value,
                                   mk.test(na.omit(Catalonia$Data),continuity = T)$p.value),each=length(getYearsAsINDEX(Catalonia))))
      
      series.indices[[i]][[m]][[s]] <- Serie
      gc()
      
    }
  }
}
save(series.indices,file=paste("series.indices.yr.CMIP6.",scenario[sc],".Rdata",sep=""))

#--------Seasonal indices--------------------#
files <- intersect(list.files(pattern="210012.nc",recursive = T),list.files(pattern="",recursive = T))
files <- intersect(files,list.files(pattern=scenario[sc],recursive=T))

seasons <- list(c(12,1,2),c(3,4,5),c(6,7,8),c(9,10,11)) #DJF #MAM #JJA #SON #ANNUAL
ssn <- c("Winter","Spring","Summer","Autumn")#,"Annual")
indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
             "rx5day","rx1day")
Model <- NA

series.indices <- list()

for(i in 1:length(indices)){
  print(indices[i])
  files_subset <- intersect(files,list.files(pattern=indices[i],recursive = T))
  
  series.indices[[i]] <- list()
  
  for(m in 1:length(files_subset)){
    print(files_subset[m])
    
    series.indices[[i]][[m]] <- list()
    
    for(s in 1:length(seasons)){
      di <- dataInventory(files_subset[m])
      
      ini <- nchar(names(di))                            
      out <- nchar("_historical-ssp585_r1i1p1f1_195001-210012.nc") 
      Model[m] <- substr(files_subset[m],start=ini+33,stop=nchar(files_subset[m])-out)
      
      data <- loadGridData(files_subset[m],var=names(di),years=1950:2100)
      
      #Interpolate into a common grid and mask ocean
      data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
      for(y in 1:dim(data$Data)[1]){
        data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
      }
      
      #Estimating anomalies wrt 1981-2010 reference period
      data <- scaleGrid(data,base = subsetGrid(data,years=1981:2010),type = "center",time.frame = "monthly")
      
      data <- aggregateGrid(data,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
      
      #Serie regional
      EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
      Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
      
      Serie <- data.frame(Serie=c(EuroMediterranean$Data,Catalonia$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Region=rep(c("EuroMediterranean","Catalonia"),each=length(getYearsAsINDEX(Catalonia))),
                          Year=rep(getYearsAsINDEX(Catalonia),2),
                          Tau=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)[["estimates"]][["tau"]],
                                    mk.test(na.omit(Catalonia$Data),continuity = T)[["estimates"]][["tau"]]),each=length(getYearsAsINDEX(Catalonia))),
                          pv=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)$p.value,
                                   mk.test(na.omit(Catalonia$Data),continuity = T)$p.value),each=length(getYearsAsINDEX(Catalonia))))
      
      series.indices[[i]][[m]][[s]] <- Serie
      gc()
      
    }
  }
}
save(series.indices,file=paste("series.indices.mon.CMIP6.",scenario[sc],".Rdata",sep=""))
}
