library(ggplot2)
library(ncdf4)
library(trend)
library(loadeR)
library(transformeR)
library(visualizeR)
library(RColorBrewer)

rm(list=ls()) ; gc()

#Working directory
dir <- "/esarchive/scratch/molmo/CLIMCAT_MJ/CORDEX/"
OUTPUTS <- "/esarchive/scratch/molmo/CLIMCAT_MJ/OUTPUTS2/"
setwd(dir)

invariable <- loadGridData(dataset = "/esarchive/scratch/molmo/mask_0.1_EUROCORDEX.nc",
                           var="sftlf") 

#interpGrid(invariable,new.coordinates = list(x=seq(-9.5,24.5,by=0.1),y=seq(33.5,44.5,by=0.1))) %>%
invariable <-  binaryGrid(invariable,condition = "GT",threshold=0)

#-------Annual indices---------#
#Useful information
indices <- c("wsdi","csdi","tr",
             "r20mm","prcptot","cdd","cwd")

files <- intersect(list.files(pattern="2100.nc",recursive = T),list.files(pattern="yr",recursive = T))
files <- intersect(files,list.files(pattern="rcp45",recursive=T))
files2 <- intersect(list.files(pattern="2099.nc",recursive=T),list.files(pattern="yr",recursive=T))
files2 <- intersect(files2,list.files(pattern="rcp45",recursive=T))
files3 <- intersect(list.files(pattern="ERA5_NULL_NULL_1950-2022.nc",recursive=T),list.files(pattern="yr",recursive=T))

files <- c(files,files2,files3)

ssn <- "Annual"
seasons <- list(c(1:12))
Model <- NA

trend.indices <- list()
series.indices <- list()

for(i in 1:length(indices)){
  print(indices[i])
  files_subset <- intersect(files,list.files(pattern=indices[i],recursive=T))
  trend.indices[[i]] <- list()
  series.indices[[i]] <- list()
  
  for(m in 1:length(files_subset)){
    print(files_subset[m])
    
    trend.indices[[i]][[m]] <- list()
    series.indices[[i]][[m]] <- list()
    
    for(s in 1:length(seasons)){
      di <- dataInventory(files_subset[m])
      
      ini <- nchar(names(di))                            
      out <- nchar("_historical-rcp45_r1i1p1_1950-2100.nc") 
      Model[m] <- substr(files_subset[m],start=ini+32,stop=nchar(files_subset[m])-out)
      
      data <- loadGridData(files_subset[m],var=names(di),
                           years=as.numeric(substr(di[[names(di)]]$Dimensions$time$Date_range,1,4)):2005)
      
      #Interpolate into a common grid and mask ocean
      data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
      for(y in 1:dim(data$Data)[1]){
        data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
      }
      
      trends <- matrix(NA,length(data$xyCoords$y),length(data$xyCoords$x))
      tau <- matrix(NA,length(data$xyCoords$y),length(data$xyCoords$x))
      signif <- matrix(NA,dim(trends)[1],dim(trends)[2])
      
      gc()
      
      for(j in 1:dim(trends)[1]){
        for(k in 1:dim(trends)[2]){
          if(!is.na(mean(data$Data[,j,k],na.rm=T))){
            trends[j,k] <- lm(data$Data[,j,k]~c(1:length(unique(getYearsAsINDEX(data)))))[["coefficients"]][2]
            tau[j,k] <- mk.test(na.omit(data$Data[,j,k]),continuity = T)[["estimates"]][["tau"]]
            signif[j,k] <- mk.test(na.omit(data$Data[,j,k]),continuity = T)$p.value
          }
        }
      }
      
      datita <- data.frame(Trend=as.numeric(trends)*10/length(unique(getYearsAsINDEX(data))),
                           Tau=as.numeric(tau),
                           Significance=as.numeric(signif),
                           Season=rep(ssn[s],length(as.numeric(trends))),
                           Model=rep(Model[m],length(as.numeric(trends))),
                           Longitude=rep(data$xyCoords$x,each=length(data$xyCoords$y)),
                           Latitude=rep(data$xyCoords$y,length(data$xyCoords$x)),
                           Index=rep(indices[i],length(as.numeric(trends))))
      
      #Serie regional
      EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
      Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
      
      Serie <- data.frame(Serie=c(EuroMediterranean$Data,Catalonia$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Region=rep(c("EuroMediterranean","Catalonia"),each=length(getYearsAsINDEX(Catalonia))),
                          Year=rep(getYearsAsINDEX(Catalonia),2),
                          Tau=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)[["estimates"]][["tau"]],
                                    mk.test(na.omit(Catalonia$Data),continuity = T)[["estimates"]][["tau"]]),each=length(getYearsAsINDEX(Catalonia))),
                          pv=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)$p.value,
                                   mk.test(na.omit(Catalonia$Data),continuity = T)$p.value),each=length(getYearsAsINDEX(Catalonia))))
      
      trend.indices[[i]][[m]][[s]] <- datita #tal vez acá va el data.frame
      series.indices[[i]][[m]][[s]] <- Serie
      gc()
      
      if(indices[i]%in%c("cdd")){paleta=rev(brewer.pal(11,"BrBG")); range=c(-0.2,0.2)}
      if(indices[i]%in%c("cwd","r20mm")){paleta=c(brewer.pal(11,"BrBG")); range=c(-0.2,0.2)}
      if(indices[i]%in%c("prcptot")){paleta=brewer.pal(11,"BrBG"); range=c(-3,3)}
      if(indices[i]%in%c("wsdi","tr")){paleta=rev(brewer.pal(11,"RdYlBu")); range=c(-0.8,0.8)}
      if(indices[i]%in%c("csdi")){paleta=brewer.pal(11,"RdYlBu"); range=c(-0.8,0.8)}
      
      # png(filename = paste(OUTPUTS,"Trend_1950-2014_",ssn[s],"_",indices[i],"_",Model[m],".png",sep=""),res=600,width=3000,height=2000)
      # 
      # if(dim(subset(datita,Significance<0.05))[1]!=0){
      #   print(ggplot(data=datita,aes(x=Longitude,y=Latitude,fill=Trend)) + geom_contour_fill(bins = 20)  +
      #           scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]],"decade",sep="/"),colours=paleta,limits=range,na.value = "white")+
      #           coord_quickmap(xlim=c(min(datita$Longitude),max(datita$Longitude)),
      #                          ylim=c(min(datita$Latitude),max(datita$Latitude)),expand=F)+ borders() +
      #           theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],sep=" "))+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
      #   dev.off()}else{
      #     print(ggplot(data=datita,aes(x=Longitude,y=Latitude,fill=Trend)) + geom_contour_fill(bins = 20)  +
      #             scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]],"decade",sep="/"),colours=paleta,limits=range,na.value = "white")+
      #             coord_quickmap(xlim=c(min(datita$Longitude),max(datita$Longitude)),
      #                            ylim=c(min(datita$Latitude),max(datita$Latitude)),expand=F)+ borders() + theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],sep=" ")))#+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
      #   }
    }
  }
}
save(trend.indices,file="trend.indices.yr.CORDEX.historical.Rdata")
save(series.indices,file="series.indices.yr.CORDEX.historical.Rdata")

#--------Seasonal indices--------------------#
files <- intersect(list.files(pattern="210012.nc",recursive = T),list.files(pattern="mon",recursive = T))
files <- intersect(files,list.files(pattern="rcp45",recursive=T))
files2 <- intersect(list.files(pattern="209912.nc",recursive=T),list.files(pattern="mon",recursive=T))
files2 <- intersect(files2,list.files(pattern="rcp45",recursive=T))
files3 <- intersect(list.files(pattern="ERA5_NULL_NULL_195001-202212.nc",recursive=T),list.files(pattern="mon",recursive=T))

files <- c(files,files2,files3)

seasons <- list(c(12,1,2),c(3,4,5),c(6,7,8),c(9,10,11)) #DJF #MAM #JJA #SON #ANNUAL
ssn <- c("Winter","Spring","Summer","Autumn")#,"Annual")
indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
             "rx5day","rx1day")
Model <- NA

trend.indices <- list()
series.indices <- list()

for(i in 1:length(indices)){
  print(indices[i])
  files_subset <- intersect(files,list.files(pattern=indices[i],recursive = T))
  trend.indices[[i]] <- list()
  series.indices[[i]] <- list()
  
  for(m in 1:length(files_subset)){
    print(files_subset[m])
    
    trend.indices[[i]][[m]] <- list()
    series.indices[[i]][[m]] <- list()
    
    for(s in 1:length(seasons)){
      di <- dataInventory(files_subset[m])
      
      ini <- nchar(names(di))                            
      out <- nchar("_historical-ssp585_r1i1p1f1_195001-210012.nc") 
      Model[m] <- substr(files_subset[m],start=ini+33,stop=nchar(files_subset[m])-out)
      
      data <- loadGridData(files_subset[m],var=names(di),
                           years=as.numeric(substr(di[[names(di)]]$Dimensions$time$Date_range,1,4)):2005)
      
      #Interpolate into a common grid and mask ocean
      data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
      for(y in 1:dim(data$Data)[1]){
        data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
      }
      
      data <- aggregateGrid(data,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
      
      trends <- matrix(NA,length(data$xyCoords$y),length(data$xyCoords$x))
      tau <- matrix(NA,length(data$xyCoords$y),length(data$xyCoords$x))
      signif <- matrix(NA,dim(trends)[1],dim(trends)[2])
      
      for(j in 1:dim(trends)[1]){
        for(k in 1:dim(trends)[2]){
          if(!is.na(mean(data$Data[,j,k],na.rm=T))){
            trends[j,k] <- lm(data$Data[,j,k]~c(1:length(unique(getYearsAsINDEX(data)))))[["coefficients"]][2]
            tau[j,k] <- mk.test(data$Data[,j,k],continuity = T)[["estimates"]][["tau"]]
            signif[j,k] <- mk.test(data$Data[,j,k],continuity = T)$p.value
          }
        }
      }
      
      datita <- data.frame(Trend=as.numeric(trends)*10/length(unique(getYearsAsINDEX(data))),
                           Tau=as.numeric(tau),
                           Significance=as.numeric(signif),
                           Season=rep(ssn[s],length(as.numeric(trends))),
                           Model=rep(Model[m],length(as.numeric(trends))),
                           Longitude=rep(data$xyCoords$x,each=length(data$xyCoords$y)),
                           Latitude=rep(data$xyCoords$y,length(data$xyCoords$x)),
                           Index=rep(indices[i],length(as.numeric(trends))))
      
      #Serie regional
      EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
      Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
      
      Serie <- data.frame(Serie=c(EuroMediterranean$Data,Catalonia$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Region=rep(c("EuroMediterranean","Catalonia"),each=length(getYearsAsINDEX(Catalonia))),
                          Year=rep(getYearsAsINDEX(Catalonia),2),
                          Tau=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)[["estimates"]][["tau"]],
                                    mk.test(na.omit(Catalonia$Data),continuity = T)[["estimates"]][["tau"]]),each=length(getYearsAsINDEX(Catalonia))),
                          pv=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)$p.value,
                                   mk.test(na.omit(Catalonia$Data),continuity = T)$p.value),each=length(getYearsAsINDEX(Catalonia))))
      
      trend.indices[[i]][[m]][[s]] <- datita #tal vez acá va el data.frame
      series.indices[[i]][[m]][[s]] <- Serie
      gc()
      
      # if(di[[names(di)]][["Units"]]%in%c("mm")){paleta=brewer.pal(11,"BrBG"); range=c(-0.25,0.25)}
      # if(di[[names(di)]][["Units"]]%in%c("degrees_C","days","%")){paleta=rev(brewer.pal(11,"RdBu")); range=c(-0.4,0.4)}
      # 
      # png(filename = paste(OUTPUTS,"Trend_1950_2014_",ssn[s],"_",indices[i],"_",Model[m],".png",sep=""),res=600,width=3000,height=2000)
      # if(dim(subset(datita,Significance<0.05))[1]!=0){
      #   print(ggplot(data=datita,aes(x=Longitude,y=Latitude,fill=Trend)) + geom_contour_fill(bins = 20)  +  
      #           scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]],"decade",sep="/"),colours=paleta,limits=range,na.value = "white")+
      #           coord_quickmap(xlim=c(min(datita$Longitude),max(datita$Longitude)),
      #                          ylim=c(min(datita$Latitude),max(datita$Latitude)),expand=F)+ borders() + 
      #           theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],sep=" "))+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
      #   dev.off()}else{
      #     print(ggplot(data=datita,aes(x=Longitude,y=Latitude,fill=Trend)) + geom_contour_fill(bins = 20)  +  
      #             scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]],"decade",sep="/"),colours=paleta,limits=range,na.value = "white")+
      #             coord_quickmap(xlim=c(min(datita$Longitude),max(datita$Longitude)),
      #                            ylim=c(min(datita$Latitude),max(datita$Latitude)),expand=F)+ borders() + 
      #             theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],sep=" ")))#+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
      #     
      #   }
    }
  }
}
save(trend.indices,file="trend.indices.mon.CORDEX.historical.Rdata")
save(series.indices,file="series.indices.mon.CORDEX.historical.Rdata")

#--------Ploteo de heatmap-------#

#Annual
load("series.indices.yr.CORDEX.historical.Rdata")
indices <- c("wsdi","csdi","tr",
             "r20mm","prcptot","cdd","cwd")

test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(series.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",series.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

test3 <- do.call("rbind",test2)
test3$Model[which(test3$Model=="")] <- "ERA5"

Model <- unique(test3$Model)
Model <- Model[order(Model)]

test3$Model <- factor(test3$Model,levels=rev(c("ERA5",Model[-which(Model=="ERA5")])))
test3$Index <- toupper(test3$Index)
test3$Index <- factor(test3$Index,levels=toupper(c("cdd","cwd","prcptot","r20mm","csdi","wsdi","tr")))
#trend <- aggregate(Tau~Model+Index,test3,FUN="mean",na.rm=T)
#trend$Significance <- sign$Significance
#trend$Model[which(trend$Model=="")] <- "ERA5"

png(filename = paste(OUTPUTS,"Heatmap_CORDEX_Tau_Annual_Trends_1950-2005",".png",sep=""),res=300,width=3000,height=2000)
print(
  ggplot(data=test3,aes(x=Index,y=Model,z=Tau)) + theme_bw() + geom_contour_fill(bins = 20) + coord_cartesian(expand=F)+
    scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdYlBu")),limits=c(-0.62,0.62),
                          aesthetics = "fill",na.value = "white") + ggtitle("Annual")+ facet_wrap(~Region,nrow = 1)+
    geom_text(data=subset(test3,pv<=0.05),aes(x=Index,y=Model,label="*",fontface="bold"))
)
dev.off()

#---------------------------#
#Seasonal
load("series.indices.mon.CORDEX.historical.Rdata")
indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
             "rx5day","rx1day")

test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(series.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",series.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

test3 <- do.call("rbind",test2)
test3$Index <- toupper(test3$Index)
test3$Model[which(test3$Model=="")] <- "ERA5"

Model <- unique(test3$Model)
Model <- Model[order(Model)]

test3$Model <- factor(test3$Model,levels=rev(c("ERA5",Model[-which(Model=="ERA5")])))
test3$Index <- factor(test3$Index,levels=toupper(c("tnn","tnx","txn","txx","tn10p","tn90p","tx10p","tx90p",
                                                   "rx1day","rx5day")))

png(filename = paste(OUTPUTS,"Heatmap_CORDEX_Tau_Seasonal_Trends_1950-2005",".png",sep=""),res=300,width=4000,height=3000)
print(
  ggplot(data=test3,aes(x=Index,y=Model,z=Tau)) + theme_bw() + geom_contour_fill(bins = 20) + coord_cartesian(expand=F)+
    scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdYlBu")),limits=c(-0.65,0.65),
                          aesthetics = "fill",na.value = "white") + ggtitle("Seasonal")+ facet_grid(Season~Region)+
    geom_text(data=subset(test3,pv<=0.05),aes(x=Index,y=Model,label="*",fontface="bold"))
)
dev.off()

######################################################
#--------------------Ensemble map----------------###
agreement <- function(x){
  sig <- sign(x)
  pos <- length(which(sig>0))/length(sig)*100
  neg <- length(which(sig<0))/length(sig)*100
  value <- max(pos,neg)
  if(value<50){ag <- 0}
  if(value>=50 & value<80){ag <- 1}
  if(value>=80){ag <- 2}
  return(ag)
}


#Annual
load("trend.indices.yr.CORDEX.historical.Rdata")
indices <- c("wsdi","csdi","tr",
             "r20mm","prcptot","cdd","cwd")

test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(trend.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",trend.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

test3 <- do.call("rbind",test2)
test3$Model[which(test3$Model=="")] <- "ERA5"
Model <- unique(test3$Model)
test3$Model <- factor(test3$Model,levels=c("ERA5",Model[-which(Model=="")]))
test3$Index <- factor(test3$Index,levels=c("cdd","cwd","r20mm","prcptot","csdi","wsdi","tr"))

ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)

ensemble2 <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "agreement")
ensemble$Agreement <- ensemble2$Tau
ensemble$Tau[which(ensemble$Index=="cdd")] <- ensemble$Tau[which(ensemble$Index=="cdd")]*-1
ensemble$Tau[which(ensemble$Tau>0.2)] <- 0.19
ensemble$Tau[which(ensemble$Tau< -0.2)] <- -0.19

coarse.grid <- list(x=seq(-9.5,24.5,by=1),y=seq(33.5,44.5,by=1))
ensemble2 <- ensemble
ensemble2 <- ensemble2[which(ensemble2$Latitude%in%coarse.grid$y & ensemble2$Longitude%in%coarse.grid$x),]

p <- ggplot(data=subset(ensemble,Index%in%c("cdd","cwd","r20mm","prcptot")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("CORDEX ensemble: annual indices")+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("cdd","cwd","r20mm","prcptot")),size=subset(ensemble2,Agreement>0 & Index%in%c("cdd","cwd","r20mm","prcptot"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))

pcat <- ggplot(data=subset(ensemble,Index%in%c("cdd","cwd","r20mm","prcptot")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(0,4),
                 ylim=c(40,43),expand=F)+ borders() + 
  theme_bw() + ggtitle("CORDEX ensemble: annual indices")+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("cdd","cwd","r20mm","prcptot")),size=subset(ensemble2,Agreement>0 & Index%in%c("cdd","cwd","r20mm","prcptot"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))

###ERA5---#
ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
ensemble$Tau[which(ensemble$Index=="cdd")] <- ensemble$Tau[which(ensemble$Index=="cdd")]*-1
ensemble$Tau[which(ensemble$Tau>0.2)] <- 0.19
ensemble$Tau[which(ensemble$Tau< -0.2)] <- -0.19

h <- ggplot(data=subset(ensemble,Index%in%c("cdd","cwd","r20mm","prcptot")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20) + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

hcat <- ggplot(data=subset(ensemble,Index%in%c("cdd","cwd","r20mm","prcptot")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20) + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(0,4),
                 ylim=c(40,43),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

#temperature
ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)

ensemble2 <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "agreement")
ensemble$Agreement <- ensemble2$Tau
ensemble$Tau[which(ensemble$Index=="cdd")] <- ensemble$Tau[which(ensemble$Index=="cdd")]*-1
ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.39
ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.39

coarse.grid <- list(x=seq(-9.5,24.5,by=1),y=seq(33.5,44.5,by=1))
ensemble2 <- ensemble
ensemble2 <- ensemble2[which(ensemble2$Latitude%in%coarse.grid$y & ensemble2$Longitude%in%coarse.grid$x),]

g <- ggplot(data=subset(ensemble,Index%in%c("csdi","wsdi","tr")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("CORDEX ensemble: annual indices")+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("csdi","wsdi","tr")),size=subset(ensemble2,Agreement>0 & Index%in%c("csdi","wsdi","tr"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))

gcat <- ggplot(data=subset(ensemble,Index%in%c("csdi","wsdi","tr")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(0,4),
                 ylim=c(40,43),expand=F)+ borders() + 
  theme_bw() + ggtitle("CORDEX ensemble: annual indices")+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("csdi","wsdi","tr")),size=subset(ensemble2,Agreement>0 & Index%in%c("csdi","wsdi","tr"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))

###ERA5---#
ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
ensemble$Tau[which(ensemble$Index=="cdd")] <- ensemble$Tau[which(ensemble$Index=="cdd")]*-1
ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.39
ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.39

i <- ggplot(data=subset(ensemble,Index%in%c("csdi","wsdi","tr")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

icat <- ggplot(data=subset(ensemble,Index%in%c("csdi","wsdi","tr")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(0,4),
                 ylim=c(40,43),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

png(filename = paste(OUTPUTS,"Indices_CORDEX_Trends_1950-2005_Map_Annual_EuroMediterranean",".png",sep=""),res=300,width=4000,height=2000)
print(
  grid.arrange(p,h,g,i,nrow=2)
)
dev.off() 

png(filename = paste(OUTPUTS,"Indices_CORDEX_Trends_1950-2005_Map_Annual_Catalonia",".png",sep=""),res=300,width=4000,height=2000)
print(
  grid.arrange(pcat,hcat,gcat,icat,nrow=2)
)
dev.off() 

rm(Catalonia,data,datita,EuroMediterranean,invariable,Serie,series.indices) ; gc()

#---------------------------------------------------------#
#Seasonal
load("trend.indices.mon.CORDEX.historical.Rdata")
ssn <- c("Winter","Spring","Summer","Autumn")#,"Annual")
indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
             "rx5day","rx1day")
gc()
test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(trend.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",trend.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

gc()
test3 <- do.call("rbind",test2)
test3$Model[which(test3$Model=="")] <- "ERA5"
Model <- unique(test3$Model)
test3$Model <- factor(test3$Model,levels=c("ERA5",Model[-which(Model=="")]))
test3$Index <- factor(test3$Index,levels=c("tnn","tnx","txn","txx","tn10p","tn90p","tx10p","tx90p",
                                           "rx1day","rx5day"))
tes.t <- test3

rm(test,test2) ; gc()

for(j in 1:length(ssn)){
  test3 <- subset(tes.t,Season==ssn[j])
  
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  ensemble$Tau[which(ensemble$Tau>0.2)] <- 0.19
  ensemble$Tau[which(ensemble$Tau< -0.2)] <- -0.19
  
  ensemble2 <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "agreement")
  ensemble$Agreement <- ensemble2$Tau
  
  coarse.grid <- list(x=seq(-9.5,24.5,by=1),y=seq(33.5,44.5,by=1))
  ensemble2 <- ensemble
  ensemble2 <- ensemble2[which(ensemble2$Latitude%in%coarse.grid$y & ensemble2$Longitude%in%coarse.grid$x),]
  
  p <- ggplot(data=subset(ensemble,Index%in%c("rx1day","rx5day")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CORDEX ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("rx1day","rx5day")),size=subset(ensemble2,Agreement>0 & Index%in%c("rx1day","rx5day"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  pcat <- ggplot(data=subset(ensemble,Index%in%c("rx1day","rx5day")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CORDEX ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("rx1day","rx5day")),size=subset(ensemble2,Agreement>0 & Index%in%c("rx1day","rx5day"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  ###ERA5---#
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  ensemble$Tau[which(ensemble$Tau>0.2)] <- 0.19
  ensemble$Tau[which(ensemble$Tau< -0.2)] <- -0.19
  
  h <- ggplot(data=subset(ensemble,Index%in%c("rx1day","rx5day")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20) + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  hcat <- ggplot(data=subset(ensemble,Index%in%c("rx1day","rx5day")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20) + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  
  #temperature
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.39
  ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.39
  
  ensemble2 <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "agreement")
  ensemble$Agreement <- ensemble2$Tau
  
  coarse.grid <- list(x=seq(-9.5,24.5,by=1),y=seq(33.5,44.5,by=1))
  ensemble2 <- ensemble
  ensemble2 <- ensemble2[which(ensemble2$Latitude%in%coarse.grid$y & ensemble2$Longitude%in%coarse.grid$x),]
  
  
  g1 <- ggplot(data=subset(ensemble,Index%in%c("tnn","tnx","txn","txx")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CORDEX ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("tnn","tnx","txn","txx")),size=subset(ensemble2,Agreement>0 & Index%in%c("tnn","tnx","txn","txx"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  g2 <- ggplot(data=subset(ensemble,Index%in%c("tn10p","tn90p","tx10p","tx90p")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CORDEX ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("tn10p","tn90p","tx10p","tx90p")),size=subset(ensemble2,Agreement>0 & Index%in%c("tn10p","tn90p","tx10p","tx90p"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  g1cat <- ggplot(data=subset(ensemble,Index%in%c("tnn","tnx","txn","txx")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CORDEX ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("tnn","tnx","txn","txx")),size=subset(ensemble2,Agreement>0 & Index%in%c("tnn","tnx","txn","txx"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  g2cat <- ggplot(data=subset(ensemble,Index%in%c("tn10p","tn90p","tx10p","tx90p")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CORDEX ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble2,Agreement>0 & Index%in%c("tn10p","tn90p","tx10p","tx90p")),size=subset(ensemble2,Agreement>0 & Index%in%c("tn10p","tn90p","tx10p","tx90p"))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  ###ERA5---#
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.39
  ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.39
  
  i <- ggplot(data=subset(ensemble,Index%in%c("tnn","tnx","txn","txx")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  j1 <- ggplot(data=subset(ensemble,Index%in%c("tn10p","tn90p","tx10p","tx90p")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  icat <- ggplot(data=subset(ensemble,Index%in%c("tnn","tnx","txn","txx")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  j1cat <- ggplot(data=subset(ensemble,Index%in%c("tn10p","tn90p","tx10p","tx90p")),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins = 20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  
  png(filename = paste(OUTPUTS,"Indices_CORDEX_TX_TN_Trends_1950-2005_Map_",ssn[j],"_EuroMediterranean.png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(#p,h,
      g1,i,
      g2,j1,
      nrow=2)
  )
 dev.off() 
  
  png(filename = paste(OUTPUTS,"Indices_CORDEX_PR_Trends_1950-2005_Map_",ssn[j],"_EuroMediterranean.png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(p,h,p,h,
                 nrow=2)
  )
  dev.off() 
  gc()
  
  png(filename = paste(OUTPUTS,"Indices_CORDEX_TX_TN_Trends_1950-2005_Map_",ssn[j],"_Catalonia.png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(#p,h,
      g1cat,icat,
      g2cat,j1cat,
      nrow=2)
  )
  dev.off() 
  
  png(filename = paste(OUTPUTS,"Indices_CORDEX_PR_Trends_1950-2005_Map_",ssn[j],"_Catalonia.png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(pcat,hcat,pcat,hcat,
                 nrow=2)
  )
  dev.off() 
  gc()
}
