library(loadeR)
library(transformeR)
library(climate4R.climdex)
library(loadeR.2nc)
library(tidyr)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("~/Documents/")
OUTPUTS <- "ETCCDI_era5/"

#ETCCDI Indices calculation

###########################-tasmin-######################################
#Load observations
message("Loading tasmin observations...")
eobs <- loadGridData(dataset = "era5_tasmin_5021_masked.nc",
                      var="tasmin",
                      lonLim =c(-9.5,4.5),latLim = c(36,44)) ; gc()
num=c(1:length(eobs$xyCoords$x))
eobs$Dates$start <- as.character(seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day"))
eobs$Dates$end <- as.character(seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day"))
eobs <- gridArithmetics(eobs,273,operator="-")

message("Done.")

indices <- list(monthly=c("TNx","TNn","TN10p","TN90p"),
                annual=c("TNx","TNn","TN10p","TN90p"),
                none=c("TR","CSDI"))
frequency <- c("monthly","annual","annual")

for(i in 1:length(indices)){
index <- indices[[i]]

  for(j in 1:length(index)){

    message("Estimating ...")
    x <- lapply(num, function(num) {
    y = subsetDimension(eobs,dimension = "lon",indices = num)
    if(frequency[i]=="annual2"){
      y2 = climdexGrid(index.code = index[j],tn = y,
                       input.arg.list = list(base.range=c(1981,2010))) ; gc()
    }else{
      y2 = climdexGrid(index.code = index[j],tn = y,index.arg.list = list(freq=frequency[i]),
                       input.arg.list = list(base.range=c(1981,2010))) ; gc()
    }
    return(y2)
    gc()
    }) %>% bindGrid(dimension = "lon") %>% redim(drop=T)

    grid2nc(x,NetCDFOutFile = paste(OUTPUTS,"ETCCDI_era5_",index[j],"_",frequency[i],".nc",sep=""))
message("Done and saved.")

rm(x) ; gc()
}}

rm(eobs) ; gc()

###########################-tasmax-######################################
#Load observations
message("Loading tasmax observations...")
eobs <- loadGridData(dataset = "era5_tasmax_5021_masked.nc",
                     var="tasmax",
                     lonLim =c(-9.5,4.5),latLim = c(36,44)) ; gc()
num=c(1:length(eobs$xyCoords$x))
eobs$Dates$start <- as.character(seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day"))
eobs$Dates$end <- as.character(seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day"))
eobs <- gridArithmetics(eobs,273,operator="-")

message("Done.")

indices <- list(monthly=c("TXx","TXn","TX10p","TX90p"),
                annual=c("TXx","TXn","TX10p","TX90p"),
                none="WSDI")
frequency <- c("monthly","annual","annual")

for(i in 1:length(indices)){
  index <- indices[[i]]

  for(j in 1:length(index)){

    message("Estimating ...")
    x <- lapply(num, function(num) {
      y = subsetDimension(eobs,dimension = "lon",indices = num)
      if(frequency[i]=="annual2"){
        y2 = climdexGrid(index.code = index[j],tx = y,
                         input.arg.list = list(base.range=c(1981,2010))) ; gc()
      }else{
        y2 = climdexGrid(index.code = index[j],tx = y,index.arg.list = list(freq=frequency[i]),
                         input.arg.list = list(base.range=c(1981,2010))) ; gc()
      }
      return(y2)
      gc()
    }) %>% bindGrid(dimension = "lon") %>% redim(drop=T)

    grid2nc(x,NetCDFOutFile = paste(OUTPUTS,"ETCCDI_era5_",index[j],"_",frequency[i],".nc",sep=""))
    message("Done and saved.")

    rm(x) ; gc()
  }}

rm(eobs); gc()

###########################-precip-######################################
#Load observations
message("Loading precip observations...")
eobs <- loadGridData(dataset = "era5_prlr_5021.nc",
                     var="prlr",
                     lonLim =c(-9.5,4.5),latLim = c(36,44)) ; gc()
#eobs <- gridArithmetics(eobs,86400,operator="*") ; gc()
eobs$Dates$start <- as.character(seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day"))
eobs$Dates$end <- as.character(seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day"))
eobs <- gridArithmetics(eobs,1000,operator="*")
num=c(1:length(eobs$xyCoords$x))

message("Done.")

indices <- list(monthly=c("Rx1day","Rx5day"),
                annual=c("Rx1day","Rx5day","SDII","R10mm","R20mm","CDD","CWD","R95pTOT","R99pTOT","PRCPTOT"))
frequency <- c("monthly","annual2")

for(i in 1:length(indices)){
  index <- indices[[i]]
  
  for(j in 1:length(index)){
    
    message("Estimating ...")
    x <- lapply(num, function(num) {
      y = subsetDimension(eobs,dimension = "lon",indices = num) #%>% gridArithmetics(86400*1000,operator="*")
      if(frequency[i]=="annual2"){
        if(index[j]%in%indices$monthly){
          y2 = climdexGrid(index.code = index[j],pr = y,
                           index.arg.list = list(freq="annual"),
                           input.arg.list = list(base.range=c(1981,2010))) ; gc() 
        }else{
          y2 = climdexGrid(index.code = index[j],pr = y,
                           input.arg.list = list(base.range=c(1981,2010))) ; gc()
        }
      }else{
      y2 = climdexGrid(index.code = index[j],pr = y,index.arg.list = list(freq=frequency[i]),
                       input.arg.list = list(base.range=c(1981,2010))) ; gc()
      }
      
      return(y2)
      gc()
    }) %>% bindGrid(dimension = "lon") %>% redim(drop=T)
    
    grid2nc(x,NetCDFOutFile = paste(OUTPUTS,"ETCCDI_era5_",index[j],"_",frequency[i],".nc",sep=""))
    message("Done and saved.")
    
    rm(x) ; gc()
  }}
